/* Emperor - an X windows strategy game for 1-6 players
   Copyright (C) 1998 Frank Rustemeyer.
   This file is part of the Emperor package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   If you have changes and you want them to be included in a future
   release of Emperor, please e-mail them to mambres@mud.tap.de.

   This file implements the dialog windows.

   98/11/30, Frank Rustemeyer <mambres@mud.tap.de>: Released.
   99/01/29, Frank Rustemeyer <mambres@mud.tap.de>: Added language support.
   00/06/11, Frank Rustemeyer <mambres@mud.tap.de>: Ported to QT
   04/11/27, Frank Rustemeyer <mambres@mud.tap.de>: Network support
*/

#include <string.h>
// strcpy in setupdlg::button_pressed()

#include "emp.h"
#include "empx.h"
#include "empgame.h"
#include "text.h"

#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qlistbox.h>

#include "flag1.xpm"
#include "flag2.xpm"
#include "flag3.xpm"
#include "flag4.xpm"
#include "flag5.xpm"
#include "flag6.xpm"

extern game *g;

// ****************************************************** unloadwndw *****

unloadwndw::unloadwndw(QWidget *parent, transporter *t, const char *name) :
        QDialog(parent, name, TRUE)
{   unit *u;
    listboxunit *lbu;

    transp = t;

    setCaption(T_FERRY);
    resize(130, 250);

    auswahl = new QLabel(this);
    auswahl->move(10, 0);

    ladung = new QListBox(this); 
    ladung->move(10,30);
    ladung->resize(110, 200);
    connect(ladung, SIGNAL(selected(QListBoxItem *)), this, SLOT(select_unit(QListBoxItem *)));

    u = (unit *) FIRSTMEMBER(g->units);
    if (!u) return;
    while (u)
    {   if (u->getcarried() == t->getlfdnr())
           lbu = new listboxunit(ladung, u); 
        if (u->getlfdnr() == t->getnextunload())
           auswahl->setText(u->name());   
        u = (unit *) u->next();
        }
    }

unloadwndw::~unloadwndw(void)
{   delete ladung;
    delete auswahl;
    }

void unloadwndw::select_unit(QListBoxItem *qlbi)
{   class unit *u;
    u = ((listboxunit *) qlbi)->get_unit();
    transp->setnextunload(u->getlfdnr());
    auswahl->setText(u->name()); 
    }

// ****************************************************** setupwndw ******

setupwndw::setupwndw(QWidget *parent, const char * name) : 
     QDialog(parent, name, TRUE) 
{   int i;
    QPushButton *okbutton, *netbutton;
    QPixmap *qpx;

    resize(600, 315);
    setCaption("Emperor: Setup");

    okbutton = new QPushButton("OK", this);
    okbutton->move(150, 260);
    connect(okbutton, SIGNAL(clicked()), this, SLOT(finish_dialog()));
    netbutton = new QPushButton(T_NETWORK, this);
    netbutton->move(340, 260);
    connect(netbutton, SIGNAL(clicked()), this, SLOT(net_setup()));

    for (i = 0; i < 6; i++)
    {   switch(i)
        {   case 0: qpx = new QPixmap((const char **) flag1); break;
            case 1: qpx = new QPixmap((const char **) flag2); break;
            case 2: qpx = new QPixmap((const char **) flag3); break;
            case 3: qpx = new QPixmap((const char **) flag4); break;
            case 4: qpx = new QPixmap((const char **) flag5); break;
            case 5: qpx = new QPixmap((const char **) flag6); break;
            }
        qlb[i] = new QLabel(this);
        qlb[i]->setPixmap(*qpx);
        qlb[i]->move(30, 17+40*i);
        delete qpx;
        playerbox[i] = new QCheckBox(T_PLAYING, this);
        playerbox[i]->setChecked(g->sp[i+1].used);
        playerbox[i]->move(65, 17 + 40*i);
        connect(playerbox[i], SIGNAL(toggled(bool)), this, SLOT(player_selected()));
        compbox[i] = new QCheckBox(T_COMPUTER, this);
        compbox[i]->setChecked(g->sp[i+1].computer);
        compbox[i]->move(165, 17 + 40*i);
        connect(compbox[i], SIGNAL(toggled(bool)), this, SLOT(comp_selected()));
        playername[i] = new QLineEdit(g->sp[i+1].name, this, T_NAME);
        playername[i]->move(265, 20 + 40*i);
        playername[i]->setMaxLength(39);
        playername[i]->resize(300,22);
        }
    }

setupwndw::~setupwndw()
{   int i;
    
    for (i = 0; i < 6; i++)
    {   delete qlb[i];
        delete playerbox[i];
        delete compbox[i];
        delete playername[i];
        }
    }

void setupwndw::finish_dialog() 
{   int i, players;
 
    players = 0;
    for( i = 0; i < 6; i++)
    {   g->sp[i+1].used = playerbox[i]->isChecked();
        if (g->sp[i+1].used && !(g->sp[i+1].ext)) players++;
        g->sp[i+1].computer = compbox[i]->isChecked(); 
        strncpy(g->sp[i+1].name, playername[i]->text(), 40);
        } 
    if (!players)
    {   QMessageBox::warning(this, "Error", T_NOPLAYERS, "OK"); 
        return;
        } 
    accept();
    return;
    }
  
void setupwndw::net_setup()
{   int i;
    netwndw *net;

    for( i = 0; i < 6; i++)
       strcpy(g->sp[i+1].name, playername[i]->text());

    net = new netwndw(this);
       net->exec();
    delete net;
    }

void setupwndw::player_selected()
{   int i;
    for ( i = 0; i < 6; i++)
        if (!(playerbox[i]->isChecked()))
           compbox[i]->setChecked(0);
    }

void setupwndw::comp_selected()
{   int i;
    for ( i = 0; i < 6; i++)
        if (compbox[i]->isChecked())
           playerbox[i]->setChecked(1);
    }    

// ****************************************************** netwndw ******

netwndw::netwndw(QWidget *parent, const char * name) : 
     QDialog(parent, name, TRUE)
{   int i;
    QPushButton *okbutton;
    QPixmap *qpx;
    QLabel *ql;
    char t[10];

    resize(600, 315);
    setCaption(T_NETWORKSETUP);

    okbutton = new QPushButton("OK", this);
    okbutton->move(465, 260);
    connect(okbutton, SIGNAL(clicked()), this, SLOT(button_pressed()));

    ql = new QLabel(this);
    ql->move(30, 260);
    ql->setText(T_MYPORT);

    sprintf(t, "%d", g->get_port());
    myport = new QLineEdit(t, this, T_PORT);
    myport->move(155, 265);
    myport->setMaxLength(6);
    myport->resize(100,22);

    for (i = 0; i < 6; i++)
    {   switch(i)
        {   case 0: qpx = new QPixmap((const char **) flag1); break;
            case 1: qpx = new QPixmap((const char **) flag2); break;
            case 2: qpx = new QPixmap((const char **) flag3); break;
            case 3: qpx = new QPixmap((const char **) flag4); break;
            case 4: qpx = new QPixmap((const char **) flag5); break;
            case 5: qpx = new QPixmap((const char **) flag6); break;
            }
        qlb[i] = new QLabel(this);
        qlb[i]->setPixmap(*qpx);
        qlb[i]->move(30, 17+40*i);
        delete qpx;

        netbox[i] = new QCheckBox(T_EXTERNAL, this);
        netbox[i]->setChecked(g->sp[i+1].ext);
        netbox[i]->move(65, 17 + 40*i);

        destination[i] = new QLineEdit(g->sp[i+1].host, this, T_HOST);
        destination[i]->move(155, 20 + 40*i);
        destination[i]->setMaxLength(99);
        destination[i]->resize(300,22);

        sprintf(t, "%d", g->sp[i+1].port);
        port[i] = new QLineEdit(t, this, T_PORT);
        port[i]->move(465, 20 + 40*i);
        port[i]->setMaxLength(6);
        port[i]->resize(100,22);
        }
    }

netwndw::~netwndw(void)
{   int i;
    for (i = 0; i < 6; i++)
       delete netbox[i];
    delete myport;
    }

void netwndw::button_pressed()
{   int i, n;
    char t[10];

    for (i = 0; i < 6; i++)
    {   g->sp[i+1].ext = netbox[i]->isChecked();
        strncpy(g->sp[i+1].host, destination[i]->text(), 100);
        strncpy(t, port[i]->text(), 10);
        sscanf(t, "%d", &n);
        g->sp[i+1].port = n;
        }

    strncpy(t, myport->text(), 10);
    sscanf(t, "%d", &n);
    g->set_port(n);
    accept();
    } 

//************************************************** Listboxunit

listboxunit::listboxunit(QListBox *qlb, unit * u) :
       QListBoxText(qlb)
{    unitpointer = u;
     setText(u->name());
     }

