/* Emperor - an X windows strategy game for 1-6 players
   Copyright (C) 1998 Frank Rustemeyer.
   This file is part of the Emperor package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   If you have changes and you want them to be included in a future
   release of Emperor, please e-mail them to mambres@mud.tap.de.

   This file declares the classes for the visualisation of the game.

   98/11/30, Frank Rustemeyer <mambres@mud.tap.de>: Released.
   99/02/07, Frank Rustemeyer <mambres@mud.tap.de>: Added "Forces" Window
   99/02/09, Frank Rustemeyer <mambres@mud.tap.de>: Included cities in the Forces Window
   00/06/11, Frank Rustemeyer <mambres@mud.tap.de>: Ported to QT
   04/11/27, Frank Rustemeyer <mambres@mud.tap.de>: Network support
*/

#ifndef _WNDCLASS
#define _WNDCLASS

#include <qtimer.h>
#include <qmainwindow.h>
#include <qdialog.h>
#include <qscrollview.h>
#include <qlistbox.h>

#include "empgame.h"

class QPopupMenu;
class QProgressBar;

class mainwndw : public QMainWindow     // the main window
{   Q_OBJECT
    public:
	mainwndw(QWidget *parent = 0, const char *name = 0);
        ~mainwndw(void);
        void selectcity(city *ct); //called by the city window's next button
        void invalidate(int mode);
        void setwindow(coordinate c);
        void unbird(void);
        void display_city(city *ct);
        void uncity(void);
        void unscore(void);
        void unforces(void);
        int query_waiting() { return waiting; }
        class unit *query_selected() { return selected; }
        coordinate query_selectedpos() { return selectedpos; }
        int make_selection(coordinate c);
        void make_move(coordinate c);    
        void invalidatearea(coordinate c1, coordinate c2);
        void invalidatemoves(void); /* update rest-moves string in the status bar */
    public slots:
        void nextplayer(void);
        void finish(void);
        void button_next(void);
        void button_stopp(void);
        void button_comp(void);
        void button_road(void);
        void button_mine(void);
        void button_stay(void);
        void button_info(void);
        void display_bird(void);
        void display_score(void);
        void display_armies(void);
        void file_open(void);
        void file_save(void);
	void heartbeat(void);
    private:
        void resizeEvent(QResizeEvent *);
        void settitle(void);  
        void enable(int button);
        void disable(int button);
        void select(class unit *u);
        void invalidatesquare(coordinate c);
        void findnextunit(void); // set up next-unit-button
        void busy(int b); // display hourglass
        void keyPressEvent(QKeyEvent *qke);

        class canvas *canv;
        class birdwndw *bird;
        class citywndw *ctyw;
        class scorewndw *score;
        class armywndw *forces;
        QPopupMenu *fileopts, *zugopts;
        QTimer *qtm;
        int nextbutton;
        class unit * nextunit; // for Next-Unit button
        int stoppbutton;
        int infobutton;
        int filebutton;
        int zugbutton;
        int waiting; // waiting for players in the network
        class unit *selected;
        coordinate selectedpos;
	class listen_socket *listener;
    };

class canvas : public QScrollView
{
    public:
        canvas(QWidget *parent = 0, const char *name = 0);
        ~canvas();
        void drawContents(QPainter *p, int x, int y, int w, int h);
    private:
        void contentsMousePressEvent(QMouseEvent *);
        void contentsMouseReleaseEvent(QMouseEvent *);
        class QPixmap *landxpm, *flagsxpm, *unitsxpm;
    };
 
class birdwndw : public QDialog 
{   public:
        birdwndw(QWidget *parent, const char *name = 0);
        void invalidate(void);
    private:
        void paintEvent(QPaintEvent *);
        void mousePressEvent(QMouseEvent *);
    };

class citywndw : public QDialog 
{   Q_OBJECT
    public:
        citywndw(QWidget *parent, city *c);
        ~citywndw();                            
	void setcity(city *c);
    public slots:
        void change_production(int i);
    private:
	city *ct;
        class QProgressBar *progress;
        class QListBox *unitbox;
        class QLabel *auswahl;
        class QLabel *bombing;
    };

class scorewndw : public QDialog 
{   Q_OBJECT
    public:
        scorewndw(QWidget *parent, const char * name = 0);
        ~scorewndw();     
    public slots:
        void countcities(void);
        void countarea(void);
        void countarmies(void);
    private:
        class QPushButton *citybn;
        class QPushButton *areabn;
        class QPushButton *armybn;
        class QProgressBar *score[6];
        class QLabel *flagge[6];
        int sc[7];
    };

class armywndw : public QDialog 
{   public:
        armywndw(QWidget * parent, const char * name = 0);
        ~armywndw(void);
        void update_armies(void);
    private:
        char **extract_unit(char **pixdata, int unittype);
        char **extract_land(char **pixdata, int landtype);
        void free_array(char ** array);
        class QLabel *flagge[6];
        class QLabel *einheit[UNIT_TYPES];
        class QLabel *zahl[6*UNIT_TYPES];
        class QLabel *stadtpic;
        class QLabel *stadt[6];
    };

class listboxunit : public QListBoxText
{   public:
       listboxunit(QListBox *qlb, unit * u);
       class unit * get_unit() { return unitpointer; }
    private:
       class unit * unitpointer;
    };
 
class unloadwndw : public QDialog
{   Q_OBJECT
    public:
        unloadwndw(QWidget *parent, transporter * t, const char *name = 0);
        ~unloadwndw(void);
    public slots:
        void select_unit(QListBoxItem *qlbi);
    private:
        class QLabel *auswahl;
        class QListBox *ladung; 
        transporter *transp;
    };

class setupwndw : public QDialog 
{   Q_OBJECT
    public:
	setupwndw(QWidget *parent, const char * name = 0);
        ~setupwndw();
    public slots:
        void finish_dialog();
	void net_setup();
        void player_selected();
        void comp_selected();
    private:
        class QLabel * qlb[6];
        class QCheckBox * playerbox[6];
        class QCheckBox * compbox[6];
        class QLineEdit * playername[6];
    };

class netwndw : public QDialog
{   Q_OBJECT
    public:
	netwndw(QWidget *parent, const char * name = 0);
        ~netwndw();
    public slots:
        void button_pressed();
    private:
        class QLabel * qlb[6];
        class QCheckBox * netbox[6];
        class QLineEdit * destination[6];
        class QLineEdit * port[6];
	class QLineEdit * myport;
    };

/******************  network stuff ************************/

#define BUFSIZE 200

class basic_socket
{   public:
        basic_socket();
        ~basic_socket();
        int operator ! () { return sock < 0; }
        void set_fd(int fd) { sock = fd; }
        int active() { return sock >= 0; }
        void close_me(void);
        int get_fd() { return sock; }
    protected:
        int sock;
    };

class active_socket : public basic_socket
{   public:
        active_socket();
        active_socket(const char *hname, int port);
        char * read_socket();
        void write_socket(char *s);
    private:
        char stringbuf[BUFSIZE];
        int counter;
    };

class listen_socket : public basic_socket
{   public:
        listen_socket(int port);
        int bind_me(int port);
        int start_listen();
        class active_socket * accept_connection();
    };

#endif
